using System;
using ShopCommon;

using System.Data;
using System.Data.OleDb;

namespace Shop
{
	/// <summary>
	/// Summary description for Customer.
	/// </summary>
	public class Customer
	{
		//
		// conexo usada para inserir clientes
		// necessita de estar declarada na classe por ser utilizada no mtodo de 
		// tratamento do evento RowChanged para se obter o novo cdigo de cliente
		//
		protected OleDbConnection _conn = null;

		/// <summary>
		/// pesquisa um determinado cliente por cdigo de cliente
		/// </summary>
		/// <param name="User">username de login na aplicao</param>
		/// <param name="Pass">password de login na aplicao</param>
		/// <returns>a informao do cliente</returns>
		public CustomerDataset GetByID(string User, string Pass, int CustomerID)
		{
			//
			// validar o utilizador
			//
			if( new User().Validate(User, Pass) != ShopCommon.Status.OK )
			{
				return null;
			}

			//
			// efectuar o select, retornar o recordset no parametro de output
			//
			CustomerDataset rs = (CustomerDataset) new UtilDB().GetByID("Customers", "CustomerID", CustomerID, typeof(CustomerDataset));
			//CustomerDataset rs = (CustomerDataset) new UtilDB().GetByID("Customers", "CustomerID", CustomerID, "CustomerDataset");
			/*
			CustomerDataset rs = new CustomerDataset();
			new UtilDB().GetByID("customers", "customerID", CustomerID, ref rs);
			*/

			return rs;
		}

		/// <summary>
		/// pesquisa clientes por nome
		/// </summary>
		/// <param name="User">username de login na aplicao</param>
		/// <param name="Pass">password de login na aplicao</param>
		/// <returns>informao dos clientes que obedeam ao critrio de busca</returns>
		public CustomerDataset FindByName(string User, string Pass, string NamePattern)
		{
			//
			// validar o utilizador
			//
			if( new User().Validate(User, Pass) != ShopCommon.Status.OK )
			{
				return null;
			}

			//
			// efectuar o select, retornar o recordset no parametro de output
			//
			CustomerDataset rs = (CustomerDataset) new UtilDB().FindByStringField("customers", "Name", NamePattern, typeof(CustomerDataset));

			return rs;
		}

		/// <summary>
		/// adiciona um cliente ao sistema
		/// </summary>
		/// <param name="User">username de login na aplicao</param>
		/// <param name="Pass">password de login na aplicao</param>
		/// <param name="Status">retorna o estado de sucesso da operao</param>
		/// <returns>o cdigo de identificao do novo cliente criado ou -1 em caso de erro</returns>
		public int AddCustomer(string User, string Pass, string Name, string Address, string Phone, string Fax, string Email, out ShopCommon.Status Status)
		{
			//
			// validar o utilizador
			//
			if( new User().Validate(User, Pass) != ShopCommon.Status.OK )
			{
				Status = ShopCommon.Status.INVALID_LOGIN;
				return -1;
			}

			Status = ShopCommon.Status.NOT_OK;
			int idCust = -1;

			//
			// criar typed dataset para clientes
			//
			ShopCommon.CustomerDataset dsCust = new CustomerDataset();

			//
			// associar mtodo para tratamento de eventos
			//
			dsCust.Customers.RowChanged += new DataRowChangeEventHandler(this.Customers_RowChanged);
			
			//OleDbConnection conn = null;
			_conn = null;

			try
			{
				//
				// inserir valores no DataSet
				//
				ShopCommon.CustomerDataset.CustomersRow rowCust = dsCust.Customers.AddCustomersRow(Name, Address, Phone, Fax, Email);

				//
				// criar objecto de conexo  base de dados
				//
				_conn = new OleDbConnection(Shop.Constantes.CONN_STRING);

				//
				// criar dataadapter
				//
				System.Data.OleDb.OleDbDataAdapter da = new System.Data.OleDb.OleDbDataAdapter("SELECT * FROM Customers", _conn);

				//
				// definir comando de insero
				//
				da.InsertCommand = new OleDbCommand("INSERT INTO Customers (Name, Address, PhoneNb, FaxNb, Email) VALUES(?, ?, ?, ?, ?)", _conn);

				//
				// criar parametros e fazer bind com as colunas do DataSet
				//
				da.InsertCommand.Parameters.Add(new OleDbParameter("Name", OleDbType.VarChar, 0, "Name"));
				da.InsertCommand.Parameters.Add(new OleDbParameter("Address", OleDbType.VarChar, 0, "Address"));
				da.InsertCommand.Parameters.Add(new OleDbParameter("PhoneNb", OleDbType.VarChar, 0, "PhoneNb"));
				da.InsertCommand.Parameters.Add(new OleDbParameter("FaxNb", OleDbType.VarChar, 0, "FaxNb"));
				da.InsertCommand.Parameters.Add(new OleDbParameter("Email", OleDbType.VarChar, 0, "Email"));

				//
				// criar mapping entre as tabelas na BD e no Dataset
				//
				da.TableMappings.Add("Table", "Customers");

				//
				// confirmar alteraes para a BD
				//
				da.Update(dsCust);

				//
				// obter novo cdigo de id gerado pela BD
				//
				idCust = rowCust.CustomerID;

				Status = ShopCommon.Status.OK;
			}
			catch(System.Data.OleDb.OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO 'OLEDB' no componente 'UtilDB': " + ex);
			}
			finally
			{
				_conn.Close();
				_conn = null;
			}

			//
			// retornar cdigo de cliente gerado
			//
			return idCust;
		}

		//
		// mtodo usado para tratar eventos na insero de registos e obter o cdigo gerado
		// adaptado de:
		// http://msdn.microsoft.com/library/default.asp?url=/library/en-us/cpguide/html/cpconRetrievingIdentityOrAutonumberValues.asp
		//
		protected void Customers_RowChanged(object sender, System.Data.DataRowChangeEventArgs args)
		{
			int newID = 0;

			if (args.Action == DataRowAction.Commit)
			{
				//OleDbConnection conn = null;
				try
				{
					// Include a variable and a command to retrieve the identity value from the Access database.
					//conn = new OleDbConnection(Shop.Constantes.CONN_STRING);
					OleDbCommand idCMD = new OleDbCommand("SELECT @@IDENTITY", _conn);
					//conn.Open();

					// Retrieve the identity value and store it in the CategoryID column.
					newID = (int)idCMD.ExecuteScalar();
					args.Row["CustomerID"] = newID;
				}
				catch(System.Data.OleDb.OleDbException ex)
				{
					//
					// tratar a excepo!!!!
					//
					System.Console.WriteLine("EXCEPO 'OLEDB' no componente 'UtilDB': " + ex);
				}
				finally
				{
					//conn.Close();
				}
			}
		}

		/// <summary>
		/// adiciona um cliente ao sistema
		/// </summary>
		/// <param name="User">username de login na aplicao</param>
		/// <param name="Pass">password de login na aplicao</param>
		/// <param name="Status">retorna o estado de sucesso da operao</param>
		/// <returns>o cdigo de identificao do novo cliente criado ou -1 em caso de erro</returns>
		public int AddCustomer2(string User, string Pass, string Name, string Address, string Phone, string Fax, string Email, out ShopCommon.Status Status)
		{
			//
			// validar o utilizador
			//
			if( new User().Validate(User, Pass) != ShopCommon.Status.OK )
			{
				Status = ShopCommon.Status.INVALID_LOGIN;
				return -1;
			}

			Status = ShopCommon.Status.NOT_OK;
			int idCust = -1;

			//OleDbConnection conn = null;
			OleDbConnection conn = null;

			try
			{
				//
				// criar objecto de conexo  base de dados
				//
				conn = new OleDbConnection(Shop.Constantes.CONN_STRING);

				//
				// abrir conexo
				//
				conn.Open();

				//
				// definir comando de insero
				//
				OleDbCommand insCmd = new OleDbCommand("INSERT INTO Customers (Name, Address, PhoneNb, FaxNb, Email) VALUES(" +
					"'" + Name + "', " +
					"'" + Address + "', " +
					"'" + Phone + "', " +
					"'" + Fax + "', " +
					"'" + Email + "'" +
					")", conn
					);

				//
				// executar comando
				//
				insCmd.ExecuteNonQuery();

				//
				// definir comando de insero
				//
				OleDbCommand identityCmd = new OleDbCommand("SELECT @@IDENTITY", conn);

				//
				// executar comando
				// obter novo cdigo de id gerado pela BD
				//
				idCust = (int)identityCmd.ExecuteScalar();

				Status = ShopCommon.Status.OK;
			}
			catch(System.Data.OleDb.OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO 'OLEDB' no componente 'UtilDB': " + ex);
			}
			finally
			{
				if (conn.State == ConnectionState.Open)
					conn.Close();
			}

			//
			// retornar cdigo de cliente gerado
			//
			return idCust;
		}

		/// <summary>
		/// actualiza a informao um cliente no sistema
		/// </summary>
		/// <param name="User">username de login na aplicao</param>
		/// <param name="Pass">password de login na aplicao</param>
		/// <returns>-1 em caso de erro</returns>
		public ShopCommon.Status UpdateCustomerData(string User, string Pass, int CustomerID, string Name, string Address, string Phone, string Fax, string Email)
		{
			ShopCommon.Status status = ShopCommon.Status.NOT_OK;

			//
			// validar o utilizador
			//
			if( new User().Validate(User, Pass) != ShopCommon.Status.OK )
			{
				return ShopCommon.Status.INVALID_LOGIN;
			}

			OleDbConnection conn = null;

			try
			{
				//
				// criar objecto de conexo  base de dados
				//
				conn = new OleDbConnection(Constantes.CONN_STRING);

				//
				// construir um comando SQL para executar
				//
				string strCmd = "UPDATE customers SET name = '" + Name + "', address = '"	+ Address + "', PhoneNb = '" + Phone + "', faxNb = '"	+ Fax + "', email = '" + Email + "' WHERE customerId = " + CustomerID.ToString();

				//
				// criar comando SQL a executar
				//
				OleDbCommand cmd = new OleDbCommand(strCmd, conn);

				//
				// abrir a conexo
				//
				conn.Open();

				//
				// executar o comando e obter um DataReader
				//
				int updRows = cmd.ExecuteNonQuery();

				//
				// verificar o resultado
				//
				if (updRows == 0)
					// no actualizou registo
					status = ShopCommon.Status.NOT_OK;
				else 					
					// actualizou registo
					status = ShopCommon.Status.OK;
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no componente 'Customer': " + ex);
			}
			finally
			{
				//
				// fechar a conexo
				//
				conn.Close();
			}

			//
			// retornar valor
			//
			return status;
		}
	}
}
