using System;
using ShopCommon;

namespace Shop
{
	/// <summary>
	/// Summary description for Product.
	/// </summary>
	public class Product
	{
		/// <summary>
		/// pesquisa um determinado produto por cdigo de produto
		/// </summary>
		/// <param name="User">username de login na aplicao</param>
		/// <param name="Pass">password de login na aplicao</param>
		/// <returns>a informao do produto</returns>
		public ProductDataset GetByID(string User, string Pass, int ProductID)
		{
			//
			// validar o utilizador
			//
			if( new User().Validate(User, Pass) != ShopCommon.Status.OK )
			{
				return null;
			}

			//
			// efectuar o select, retornar o recordset no parametro de output
			//
			ProductDataset rs = (ProductDataset) new UtilDB().GetByID("Products", "ProductID", ProductID, typeof(ProductDataset));

			return rs;
		}

		/// <summary>
		/// pesquisa produtos por nome
		/// </summary>
		/// <param name="User">username de login na aplicao</param>
		/// <param name="Pass">password de login na aplicao</param>
		/// <returns>informao dos produtos que obedeam ao critrio de busca</returns>
		public ProductDataset FindByName(string User, string Pass, string NamePattern)
		{
			//
			// validar o utilizador
			//
			if( new User().Validate(User, Pass) != ShopCommon.Status.OK )
			{
				return null;
			}

			//
			// efectuar o select, retornar o recordset no parametro de output
			//
			ProductDataset rs = (ProductDataset) new UtilDB().FindByStringField("Products", "Description", NamePattern, typeof(ProductDataset));

			return rs;
		}

		/// <summary>
		/// verifica se existe uma determinada quantidade de um produto em armazem 
		/// </summary>
		/// <param name="User">username de login na aplicao</param>
		/// <param name="Pass">password de login na aplicao</param>
		/// <param name="ProductID">cdigo de produto a pesquizar</param>
		/// <param name="Quant">quantidade desejada de produto</param>
		/// <returns>cdigo de estado relativo  operao</returns></returns></returns>
		public ShopCommon.Status InStock(string User, string Pass, int ProductID, int Quant)
		{
			//
			// validar o utilizador
			//
			if( new User().Validate(User, Pass) != ShopCommon.Status.OK )
			{
				return ShopCommon.Status.INVALID_LOGIN;
			}

			return InStock(ProductID, Quant);
		}

		/// <summary>
		/// verifica se existe uma determinada quantidade de um produto em armazem.
		/// este mtodo no faz parte do interface pblico da classe, apenas  
		/// utilizado internamente na ClassLibrary
		/// </summary>
		/// <param name="ProductID">cdigo de produto a pesquizar</param>
		/// <param name="Quant">quantidade desejada de produto</param>
		/// <returns>cdigo de estado relativo  operao</returns></returns></returns>
		internal ShopCommon.Status InStock(int ProductID, int Quant)
		{
			//
			// efectuar o select
			//
			ProductDataset rs = (ProductDataset) new UtilDB().GetByID("Products", "ProductID", ProductID, typeof(ProductDataset));
			if (rs == null)
				return ShopCommon.Status.NOT_OK;

			//
			// validar stock
			//
			if (rs.Products.Rows.Count > 0)
				return (rs.Products[0].StockQtd >= Quant ? ShopCommon.Status.OK : ShopCommon.Status.INSUFICIENT_STOCK);
			else
				return ShopCommon.Status.INVALID_PRODUCT_ID;
		}
	}
}
