using System;
using System.Data;
using System.Data.OleDb;

using ShopCommon;

namespace Shop
{
	/// <summary>
	/// Summary description for Sale.
	/// </summary>
	public class Sale
	{
		/// <summary>
		/// pesquisa clientes por nome
		/// </summary>
		/// <param name="User">username de login na aplicao</param>
		/// <param name="Pass">password de login na aplicao</param>
		/// <returns>um dataset vazio para a palicao cliente preencher</returns>
		public SalesDetailDataSet CreateSalesDetail(string User, string Pass)
		{
			//
			// validar o utilizador
			//
			if( new User().Validate(User, Pass) != ShopCommon.Status.OK )
			{
				return null;
			}

			//
			// criar o dataset
			//
			SalesDetailDataSet ds = new SalesDetailDataSet();

			//
			// configurar as colunas para utilizao
			//
			ds.SaleDetails.Columns["SaleID"].AllowDBNull = true;

			return ds;
		}

		/// <summary>
		/// adiciona uma venda ao sistema
		/// </summary>
		/// <param name="User">username de login na aplicao</param>
		/// <param name="Pass">password de login na aplicao</param>
		/// <param name="RsDetails">um dataset com as linhas de detalhe da venda</param>
		/// <returns>o cdigo de status relativo  operao</returns>
		public ShopCommon.Status AddSale(string User, string Pass, int CustomerID, DateTime Date, SalesDetailDataSet RsDetails)
		{
			//
			// validar o utilizador
			//
			if( new User().Validate(User, Pass) != ShopCommon.Status.OK )
			{
				return ShopCommon.Status.INVALID_LOGIN;
			}

			//
			// validar cliente
			//
			CustomerDataset dsCust = (CustomerDataset) new UtilDB().GetByID("Customers", "CustomerID", CustomerID, typeof(CustomerDataset));
			if (dsCust.Customers.Count == 0)
				return ShopCommon.Status.INVALID_CUSTOMER_ID;

			ShopCommon.Status ret = ShopCommon.Status.NOT_OK;

			//
			// validar os produtos e quantidades pedidas
			//
			Product p = new Product();
			int[] listaProd = new int[RsDetails.SaleDetails.Rows.Count];
			int i = 0;
			foreach (SalesDetailDataSet.SaleDetailsRow regDetails in RsDetails.SaleDetails)
			{
				//
				// verificar se este produto tem quantidade suficiente em stock
				// e se existe!
				//
				ret = p.InStock(regDetails.ProductID, regDetails.Quantity);
				if (ret != ShopCommon.Status.OK)
					return ret;

				//
				// guardar o cdigo de produto para pesquisa
				//
				listaProd[i++] = regDetails.ProductID;
			}

			//
			// criar um DataSet de produtos e preencher apenas com os produtos desejados
			//
			ProductDataset pDS = (ProductDataset)new UtilDB().GetByID("Products", "ProductID", listaProd, typeof(ProductDataset));

			//
			// calcular custo por linha
			//
			foreach (SalesDetailDataSet.SaleDetailsRow regDetails in RsDetails.SaleDetails)
			{
				//
				// procurar o produto desejado
				//
				ProductDataset.ProductsRow[] row = (ProductDataset.ProductsRow[])pDS.Products.Select(regDetails.ProductID.ToString());

				//
				// calcular custo desta linha da venda
				//
				regDetails.TotalCost = regDetails.Quantity * (int)row[0]["UnitPrice"];
			}

			// ======================================
			//
			// tratar das inseres
			//
			// ======================================

			OleDbConnection conn = null;
			OleDbTransaction tx = null;
			try
			{
				//
				// criar objecto de conexo  base de dados
				//
				conn = new OleDbConnection(Shop.Constantes.CONN_STRING);

				//
				// iniciar transaco
				//
				tx = conn.BeginTransaction();

				//
				// criar typed dataset para vendas
				//
				ShopCommon.SaleDataSet dsSale = new SaleDataSet();

				//
				// inserir valores no DataSet
				//
				ShopCommon.SaleDataSet.SalesRow rowCust = dsSale.Sales.AddSalesRow(CustomerID, Date);

				//
				// criar componente com DataAdapters
				//
				SaleComponent compSale = new SaleComponent();

				//
				// associar evento para obter o ID
				//
				compSale.SalesDataAdapter.RowUpdated += new OleDbRowUpdatedEventHandler(SalesDataAdapter_RowUpdated);

				//
				// inserir registo via DataAdapter
				//
				compSale.SalesDataAdapter.InsertCommand.Transaction = tx;
				compSale.SalesDataAdapter.Update(dsSale, "Sales");

				// TODO: inserir linhas de detalhe
				// TODO: actaulizar quantidades de produto em stock

				//
				// fechar transaco
				//
				tx.Commit();
			}
			catch(Exception ex)
			{
				//
				// descartar mudanas efectuadas
				//
				tx.Rollback();
			}
			finally
			{
				conn.Close();
			}

			return ret;
		}

		private void SalesDataAdapter_RowUpdated(object sender, OleDbRowUpdatedEventArgs args)
		{
			try
			{
				// Include a variable and a command to retrieve the identity value from the Access database.
				OleDbCommand idCMD = new OleDbCommand("SELECT @@IDENTITY", args.Command.Connection);
				idCMD.Transaction = args.Command.Transaction;

				// Retrieve the identity value and store it in the CategoryID column.
				int newID = (int)idCMD.ExecuteScalar();
				args.Row["SaleID"] = newID;
			}
			catch(System.Data.OleDb.OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO 'OLEDB' na 'class Sale': " + ex);
			}
		}
	}
}
