using System;
using Shop;
using System.Data;
using System.Data.OleDb;

namespace Shop
{
	/// <summary>
	/// Summary description for UtilDB.
	/// </summary>
	public sealed class UtilDB
	{
		#region GetByID Methods
		/// <summary>
		/// Obtem um dataset resultante de uma pesquisa por chave
		/// </summary>
		/// <param name="table">tabela a pesquisar</param>
		/// <param name="field">campo chave a usar</param>
		/// <param name="key">valor da chave a pesquisar</param>
		/// <param name="TypedDataSetClassName">Nome da classe a gerar dinamicamente. s funciona para 
		/// classes definidas no assembly 'ShopCommon'</param>
		/// <returns>um dataset do tipo pedido</returns>
		public DataSet GetByID(string table, string field, int key, string TypedDataSetClassName)
		{
			OleDbConnection conn = null;
			DataSet ret = null;

			try
			{
				//
				// carregar dinamicamente o assembly onde se encontra a classe que queremos criar
				//
				System.Reflection.Assembly assb = System.Reflection.Assembly.Load("ShopCommon");				

				//
				// criar dinamicamente um objecto do tipo pretendido
				//
				ret = (DataSet)assb.CreateInstance("ShopCommon." + TypedDataSetClassName);
				if (ret == null)
					throw new System.Exception("erro ao criar instncia de <" + TypedDataSetClassName + ">");

				//
				// criar objecto de conexo  base de dados
				//
				conn = new OleDbConnection(Shop.Constantes.CONN_STRING);

				//
				// criar um Datadapter para executar o comando e devolver o dataset
				//
				IDataAdapter da = new OleDbDataAdapter("Select * from " + table + " where " + field + "=" + key.ToString(), conn);

				//
				// executar o comando e preencher um Dataset
				//
				da.Fill(ret);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO 'OLEDB' no componente 'UtilDB': " + ex);
			}
			catch(Exception ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no componente 'UtilDB': " + ex);
			}
			finally
			{
				//
				// fechar a conexo
				//
				conn.Close();
			}

			//
			// retornar valor
			//
			return ret;
		}

		/// <summary>
		/// Obtem um dataset resultante de uma pesquisa por chave
		/// </summary>
		/// <param name="table">tabela a pesquisar</param>
		/// <param name="field">campo chave a usar</param>
		/// <param name="key">valor da chave a pesquisar</param>
		/// <param name="TypedDataSetClassType">tipo da classe a gerar dinamicamente</param>
		/// <returns>um dataset do tipo pedido</returns>
		public DataSet GetByID(string table, string field, int key, Type TypedDataSetClassType)
		{
			OleDbConnection conn = null;
			DataSet ret = null;

			try
			{
				//
				// obter informao sobre o construtor do tipo que queremos criar
				//
				System.Reflection.ConstructorInfo ctor = TypedDataSetClassType.GetConstructor(System.Type.EmptyTypes);

				//
				// criar dinamicamente um objecto do tipo pretendido
				//
				ret = (DataSet)ctor.Invoke(null);
				if (ret == null)
					throw new System.Exception("erro ao criar instncia de <" + TypedDataSetClassType + ">");

				//
				// criar objecto de conexo  base de dados
				//
				conn = new OleDbConnection(Shop.Constantes.CONN_STRING);

				//
				// criar um Datadapter para executar o comando e devolver o dataset
				//
				string cmd = "Select * from " + table + " where " + field + "=" + key.ToString();
				IDataAdapter da = new OleDbDataAdapter(cmd, conn);

				//
				// executar o comando e preencher um Dataset
				//
				da.Fill(ret);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO 'OLEDB' no componente 'UtilDB': " + ex);
			}
			catch(Exception ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no componente 'UtilDB': " + ex);
			}
			finally
			{
				//
				// fechar a conexo
				//
				conn.Close();
			}

			//
			// retornar valor
			//
			return ret;
		}

		/// <summary>
		/// Obtem um dataset resultante de uma pesquisa por chave
		/// </summary>
		/// <param name="table">tabela a pesquisar</param>
		/// <param name="field">campo chave a usar</param>
		/// <param name="keys">array com valores das chaves a pesquisar</param>
		/// <param name="TypedDataSetClassType">tipo da classe a gerar dinamicamente</param>
		/// <returns>um dataset do tipo pedido</returns>
		public DataSet GetByID(string table, string field, int[] keys, Type TypedDataSetClassType)
		{
			OleDbConnection conn = null;
			DataSet ret = null;

			try
			{
				//
				// obter informao sobre o construtor do tipo que queremos criar
				//
				System.Reflection.ConstructorInfo ctor = TypedDataSetClassType.GetConstructor(System.Type.EmptyTypes);

				//
				// criar dinamicamente um objecto do tipo pretendido
				//
				ret = (DataSet)ctor.Invoke(null);
				if (ret == null)
					throw new System.Exception("erro ao criar instncia de <" + TypedDataSetClassType + ">");

				//
				// criar objecto de conexo  base de dados
				//
				conn = new OleDbConnection(Shop.Constantes.CONN_STRING);

				//
				// construir SQL para clausula WHERE
				//
				string keys_list = "(";
				foreach(int key in keys)
				{
					keys_list += key + ",";
				}
				keys_list = keys_list.Substring(0, keys_list.Length-1) + ")";

				//
				// criar um Datadapter para executar o comando e devolver o dataset
				//
				IDataAdapter da = new OleDbDataAdapter("Select * from " + table + " where " + field + " in " + keys_list, conn);

				//
				// executar o comando e preencher um Dataset
				//
				da.Fill(ret);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO 'OLEDB' no componente 'UtilDB': " + ex);
			}
			catch(Exception ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no componente 'UtilDB': " + ex);
			}
			finally
			{
				//
				// fechar a conexo
				//
				conn.Close();
			}

			//
			// retornar valor
			//
			return ret;
		}

		/// <summary>
		/// Obtem um dataset resultante de uma pesquisa por chave
		/// </summary>
		/// <param name="table">tabela a pesquisar</param>
		/// <param name="field">campo chave a usar</param>
		/// <param name="key">valor da chave a pesquisar</param>
		/// <param name="ret">objecto DataSet j criado a preencher</param>
		/// <returns>um dataset do tipo pedido</returns>
		public DataSet GetByID(string table, string field, int key, ref DataSet ret)
		{
			OleDbConnection conn = null;

			try
			{
				//
				// criar objecto de conexo  base de dados
				//
				conn = new OleDbConnection(Shop.Constantes.CONN_STRING);

				//
				// criar um Datadapter para executar o comando e devolver o dataset
				//
				IDataAdapter da = new OleDbDataAdapter("Select * from " + table + " where " + field + "=" + key.ToString(), conn);

				//
				// executar o comando e preencher um Dataset
				//
				da.Fill(ret);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO 'OLEDB' no componente 'UtilDB': " + ex);
			}
			catch(Exception ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no componente 'UtilDB': " + ex);
			}
			finally
			{
				//
				// fechar a conexo
				//
				conn.Close();
			}

			//
			// retornar valor
			//
			return ret;
		}

		/// <summary>
		/// Obtem um dataset resultante de uma pesquisa por chave
		/// </summary>
		/// <param name="table">tabela a pesquisar</param>
		/// <param name="field">campo chave a usar</param>
		/// <param name="key">valor da chave a pesquisar</param>
		/// <returns>um dataset do tipo pedido</returns>
		public DataSet GetByID(string table, string field, int key)
		{
			OleDbConnection conn = null;
			DataSet ret = new DataSet();

			try
			{
				//
				// criar objecto de conexo  base de dados
				//
				conn = new OleDbConnection(Shop.Constantes.CONN_STRING);

				//
				// criar um Datadapter para executar o comando e devolver o dataset
				//
				IDataAdapter da = new OleDbDataAdapter("Select * from " + table + " where " + field + "=" + key.ToString(), conn);

				//
				// executar o comando e preencher um Dataset
				//
				da.Fill(ret);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no componente 'UtilDB': " + ex);
			}
			finally
			{
				//
				// fechar a conexo
				//
				conn.Close();
			}

			//
			// retornar valor
			//
			return ret;
		}

#endregion
		
		#region FindByStringField Methods
		/// <summary>
		/// Obtem um dataset resultante de uma pesquisa por chave
		/// </summary>
		/// <param name="table">tabela a pesquisar</param>
		/// <param name="field">campo chave a usar</param>
		/// <param name="key">valor da chave a pesquisar</param>
		/// <param name="TypedDataSetClassName">Nome da classe a gerar dinamicamente. s funciona para 
		/// classes definidas no assembly 'ShopCommon'</param>
		/// <returns>um dataset do tipo pedido</returns>
		public DataSet FindByStringField(string table, string field, string key, string TypedDataSetClassName)
		{
			OleDbConnection conn = null;
			DataSet ret = null;

			try
			{
				//
				// carregar dinamicamente o assembly onde se encontra a classe que queremos criar
				//
				System.Reflection.Assembly assb = System.Reflection.Assembly.Load("ShopCommon");				

				//
				// criar dinamicamente um objecto do tipo pretendido
				//
				ret = (DataSet)assb.CreateInstance("ShopCommon." + TypedDataSetClassName);
				if (ret == null)
					throw new System.Exception("erro ao criar instncia de <" + TypedDataSetClassName + ">");

				//
				// criar objecto de conexo  base de dados
				//
				conn = new OleDbConnection(Shop.Constantes.CONN_STRING);

				//
				// criar um Datadapter para executar o comando e devolver o dataset
				//
				IDataAdapter da = new OleDbDataAdapter("SELECT * FROM " + table + " WHERE " + field + " LIKE '" + key + "%'", conn);

				//
				// executar o comando e preencher um Dataset
				//
				da.Fill(ret);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO 'OLEDB' no componente 'UtilDB': " + ex);
			}
			catch(Exception ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no componente 'UtilDB': " + ex);
			}
			finally
			{
				//
				// fechar a conexo
				//
				conn.Close();
			}

			//
			// retornar valor
			//
			return ret;
		}

		/// <summary>
		/// Obtem um dataset resultante de uma pesquisa por chave
		/// </summary>
		/// <param name="table">tabela a pesquisar</param>
		/// <param name="field">campo chave a usar</param>
		/// <param name="key">valor da chave a pesquisar</param>
		/// <param name="TypedDataSetClassType">tipo da classe a gerar dinamicamente</param>
		/// <returns>um dataset do tipo pedido</returns>
		public DataSet FindByStringField(string table, string field, string key, Type TypedDataSetClassType)
		{
			OleDbConnection conn = null;
			DataSet ret = null;

			try
			{
				//
				// obter informao sobre o construtor do tipo que queremos criar
				//
				System.Reflection.ConstructorInfo ctor = TypedDataSetClassType.GetConstructor(System.Type.EmptyTypes);

				//
				// criar dinamicamente um objecto do tipo pretendido
				//
				ret = (DataSet)ctor.Invoke(null);
				if (ret == null)
					throw new System.Exception("erro ao criar instncia de <" + TypedDataSetClassType + ">");

				//
				// criar objecto de conexo  base de dados
				//
				conn = new OleDbConnection(Shop.Constantes.CONN_STRING);

				//
				// criar um Datadapter para executar o comando e devolver o dataset
				//
				IDataAdapter da = new OleDbDataAdapter("SELECT * FROM " + table + " WHERE " + field + " LIKE '" + key + "%'", conn);

				//
				// executar o comando e preencher um Dataset
				//
				da.Fill(ret);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO 'OLEDB' no componente 'UtilDB': " + ex);
			}
			catch(Exception ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no componente 'UtilDB': " + ex);
			}
			finally
			{
				//
				// fechar a conexo
				//
				conn.Close();
			}

			//
			// retornar valor
			//
			return ret;
		}

		/// <summary>
		/// Obtem um dataset resultante de uma pesquisa por chave
		/// </summary>
		/// <param name="table">tabela a pesquisar</param>
		/// <param name="field">campo chave a usar</param>
		/// <param name="key">valor da chave a pesquisar</param>
		/// <param name="ret">objecto DataSet j criado a preencher</param>
		/// <returns>um dataset do tipo pedido</returns>
		public DataSet FindByStringField(string table, string field, string key, ref DataSet ret)
		{
			OleDbConnection conn = null;

			try
			{
				//
				// criar objecto de conexo  base de dados
				//
				conn = new OleDbConnection(Shop.Constantes.CONN_STRING);

				//
				// criar um Datadapter para executar o comando e devolver o dataset
				//
				IDataAdapter da = new OleDbDataAdapter("SELECT * FROM " + table + " WHERE " + field + " LIKE '" + key + "%'", conn);

				//
				// executar o comando e preencher um Dataset
				//
				da.Fill(ret);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO 'OLEDB' no componente 'UtilDB': " + ex);
			}
			catch(Exception ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no componente 'UtilDB': " + ex);
			}
			finally
			{
				//
				// fechar a conexo
				//
				conn.Close();
			}

			//
			// retornar valor
			//
			return ret;
		}

		/// <summary>
		/// Obtem um dataset resultante de uma pesquisa por chave
		/// </summary>
		/// <param name="table">tabela a pesquisar</param>
		/// <param name="field">campo chave a usar</param>
		/// <param name="key">valor da chave a pesquisar</param>
		/// <returns>um dataset do tipo pedido</returns>
		public DataSet FindByStringField(string table, string field, string key)
		{
			OleDbConnection conn = null;
			DataSet ret = new DataSet();

			try
			{
				//
				// criar objecto de conexo  base de dados
				//
				conn = new OleDbConnection(Shop.Constantes.CONN_STRING);

				//
				// criar um Datadapter para executar o comando e devolver o dataset
				//
				IDataAdapter da = new OleDbDataAdapter("SELECT * FROM " + table + " WHERE " + field + " LIKE '" + key + "%'", conn);

				//
				// executar o comando e preencher um Dataset
				//
				da.Fill(ret);
			}
			catch(OleDbException ex)
			{
				//
				// tratar a excepo!!!!
				//
				System.Console.WriteLine("EXCEPO no componente 'UtilDB': " + ex);
			}
			finally
			{
				//
				// fechar a conexo
				//
				conn.Close();
			}

			//
			// retornar valor
			//
			return ret;
		}
		#endregion
	}
}
